# Supported ROS codenames mapped to either ROS 1 or ROS 2
ROS_VERSION_MAP = {"noetic": 1, "humble": 2, "iron": 2, "jazzy": 2}

# ROS release dates since epoch
ROS_VERSION_RELEASE_DATE_MAP = {"noetic": 1590264000, "humble": 1653336000, "iron": 1684872000, "jazzy": 1716494400}
